#R code for the anaysis of case study.
#Author: Mohsen Sadatsafavi
#Date created: 03/10/2011
#Last date modified: 16/05/2014; 


#The main function generating results is main();
#note that T1, T2, and T3, as described in the text, are represented in the code by, respectively, TP, TS, and TFS variables/notations.


#Location of RCT dta file;
setwd("C:/Users/Mohsen/Documents/Projects/VOI/Project.Bedfellows/OPTIMAL/R");



#Initializes the environment (laoding RCT data, defining global variables, etc)
init<-function()
{
	#require(norm);	#package for imputation (multivariate normality);
	BS_TYPE_MULTINOMIAL<<-0;	#Regular bootstrap;
	BS_TYPE_DIRICHLET<<-1;	#Bayesian bootstrap; NOTE: the imputaiton package doesn't incorporate sample waiting, so Beysian BS cannot be used if imputation is to be done afterwards;

	TYPE_VETTED<<-0;
	TYPE_WEIGHTED<<-1;

	IM_WHERE_NO<<-0;	#Does not impute missing data; the future RCT will have the same pattern of missingness.
	IM_WHERE_OUT<<-1;	#Imputation outside the loop;
	IM_WHERE_IN_BEFORE<<-2;	#Imputation inside the loop but before bootstrap
	IM_WHERE_IN_AFTER<<-3; #Imputation inside the loop but after bootstrap

	IM_TYPE_NO<<-0;
	IM_TYPE_EM<<-1;	#Imputation of type EM;
	IM_TYPE_PS<<-2;	#Imputation of type EM;

	ADJUST_TYPE_NONE<<-0;
	ADJUST_TYPE_KS<<-1;
	ADJUST_TYPE_LINEAR<<-2;

	FUTURE_MISSING_METHOD_NO<<-0;
	FUTURE_MISSING_METHOD_SAMPLE<<-1;
	FUTURE_MISSING_METHOD_PROBABILITY<<-2;

	EFF_QALY<<-1;
	EFF_EXAC<<-2;

	temp<-read.csv("data.csv",header=TRUE,na.strings = ".")
	rct<<-temp;
		
	treatments<<-c('TFS','TP','TS');
	names(rct)<<-tolower(names(rct));

	TFS<<-1;
	TP<<-2;
	TS<<-3;

	sitef<-as.factor(rct[,"site"]);
	txgroupf<-as.factor(rct[,"txgroup"]);
	w<-rep(1,dim(rct)[1]);
	rct<<-cbind(rct,txgroupf, sitef,w);
	rct<<-preProcessRct(rct);
	wtp<<-50000;

	EVDISCOUNT<<-1.5;
}



#Formats the RCT data into a list of arrays for fast processing
preProcessRct<-function(rct)
{
	n<-dim(rct)[1];
	nArms<-length(unique(rct[,"txgroup"]));
	ns<-rep(0,nArms);
	subjects<-list();
	for(i in 1:nArms)
	{
		subjects<-c(subjects,0);	
		ns[i]<-sum(rct[,"txgroup"]==i);
		subjects[[i]]<-(1:n)[rct[,"txgroup"]==i];
	}

#Create a column if any cost data is missing;
	missingC<-rep(0,n);
	for(i in 1:13)
	{
		missingC<-missingC|is.na(rct[,paste("tc",i,sep="")]);
	}	
	missingC<-missingC*1;

#Create a column if any utility data is mising;
	missingU<-rep(0,n);
	for(i in 1:5)
	{
		missingU<-missingU|is.na(rct[,paste("utility",i,sep="")]);
	}	
	missingU<-missingU*1;

#Create a column if any exacerbation data is mising;
	missingE<-rep(0,n);
	for(i in 1:13)
	{
		missingE<-missingE|is.na(rct[,paste("exac",i,sep="")]);
	}	
	missingE<-missingE*1;
	rct<-cbind(rct,missingC,missingU,missingE);

#Calcualte the propensity scores for missing costs and utlities and exacerbation, separately;
	cPS<-calcPS(missingC ~ age+gender+sitef+txgroupf, data=rct);
	cPSGroup<-rep(0,n);
	for(i in 1:nArms)
	{
		cPSGroup[subjects[[i]]]<-calcPSGroup(cPS[subjects[[i]]]);
	}
	rct<-cbind(rct,cPSGroup);
	uPS<-calcPS(missingU ~ age+gender+sitef+txgroupf, data=rct);
	uPSGroup<-rep(0,n);
	for(i in 1:nArms)
	{
		uPSGroup[subjects[[i]]]<-calcPSGroup(uPS[subjects[[i]]]);
	}
	rct<-cbind(rct,uPSGroup);
	ePS<-calcPS(missingE ~ age+gender+sitef+txgroupf, data=rct);
	ePSGroup<-rep(0,n);
	for(i in 1:nArms)
	{
		ePSGroup[subjects[[i]]]<-calcPSGroup(ePS[subjects[[i]]]);
	}
	rct<-cbind(rct,ePSGroup);

	return(rct);
}







#Cost-effectiveness 
cea<-function(rct, nIteration=0, impute=TRUE, imputeExac=TRUE, adjusted=TRUE, exacDisUtil=FALSE, vet=TRUE)
{
	bExpectedValue<-FALSE;
	if(nIteration==0)
	{
		bExpectedValue<-TRUE;
		nIteration<-1;
	}

	n<-dim(rct)[1];
	nArms<-length(unique(rct[,"txgroup"]));
	ns<-rep(0,nArms);
	subjects<-list();
	for(i in 1:nArms)
	{
		subjects<-c(subjects,0);	
		ns[i]<-sum(rct[,"txgroup"]==i);
		subjects[[i]]<-(1:n)[rct[,"txgroup"]==i];
	}

	#rngseed(as.integer(runif(1)*10^9));

	w<-rep(1,4);
	
	for(iteration in 1:nIteration)
	{
		fRct<-rct;

#Imputes missing costs and qalys;		
		columNamesC<-paste("tc",1:13,sep="");
		tempC<-rct[,columNamesC];
		for(psGroup in 1:5)
			for(txGroup in 1:nArms)
				for(i in 1:13)
				{
					nas<-(1:n)[is.na(tempC[,i]) & rct[,'cPSGroup']==psGroup & rct[,'txgroup']==txGroup];
					if(length(nas)>0)
					{
						pool<-tempC[rct[,"txgroup"]==txGroup & rct[,"cPSGroup"]==psGroup,i];
						if(bExpectedValue==TRUE)
						{
							for(j in nas)
								tempC[j,i]<-mean(pool,na.rm=TRUE);
						}
						else
						{
							pool<-sample(pool[!is.na(pool)],length(pool),replace=TRUE);
							for(j in nas)
							{
								tempC[j,i]<-sample(pool,1);
							}
						}
					}
				}
		
		columNamesU<-paste("utility",1:5,sep="");
		tempU<-rct[,columNamesU];
		for(psGroup in 1:5)
			for(txGroup in 1:nArms)
				for(i in 1:5)
				{
					nas<-(1:n)[is.na(tempU[,i]) & rct[,'uPSGroup']==psGroup & rct[,'txgroup']==txGroup];
					if(length(nas)>0)
					{
						pool<-tempU[rct[,"txgroup"]==txGroup & rct[,"uPSGroup"]==psGroup,i];
						if(bExpectedValue==TRUE)
						{
							for(j in nas)
								tempU[j,i]<-mean(pool,na.rm=TRUE);
						}
						else
						{
							pool<-sample(pool[!is.na(pool)],length(pool),replace=TRUE);
							for(j in nas)
							{
								tempU[j,i]<-sample(pool,1);
							}
						}
					}
				}

		columNamesE<-paste("exac",1:13,sep="");
		tempE<-rct[,columNamesE];
		if(imputeExac==TRUE)
		{
			for(psGroup in 1:5)
				for(txGroup in 1:nArms)
					for(i in 1:13)
					{
						nas<-(1:n)[is.na(tempE[,i]) & rct[,'ePSGroup']==psGroup & rct[,'txgroup']==txGroup];
						if(length(nas)>0)
						{
							pool<-tempE[rct[,"txgroup"]==txGroup & rct[,"ePSGroup"]==psGroup,i];
							if(bExpectedValue==TRUE)
							{
								for(j in nas)
									tempE[j,i]<-mean(pool,na.rm=TRUE);
							}
							else
							{
								pool<-sample(pool[!is.na(pool)],length(pool),replace=TRUE);
								for(j in nas)
								{
									tempE[j,i]<-sample(pool,1);
								}
							}
						}
					}
		}

#Vetting
		if(vet==TRUE)
		{
			exac<-rep(0,3);
			for(i in 1:3)
				exac[i]<-weighted.mean(rowSums(tempE[subjects[[i]],],na.rm=TRUE)>0,rct[subjects[[i]],"w"]);
			w<-weighEffTPTS(exac[TP],exac[TS]);
			w[2]<-w[1]*weighEffTPTFS(exac[TP],exac[TFS]);
			ae<-rep(0,3);
			for(i in 1:3)
				ae[i]<-weighted.mean(rct[subjects[[i]],"ae"],rct[subjects[[i]],"w"]);
			w[3]<-w[2]*weighAE(ae[TP],ae[TS]);
			w[4]<-w[3]*weighAE(ae[TS],ae[TFS]);
		}
	

#Adjust for utility1
		costs<-rowSums(tempC);
		qalys<-(rowSums(tempU)-tempU[,1]/2-tempU[,5]/2)/4;

		if(exacDisUtil==TRUE)
			qalys<-qalys-rowSums(tempU,na.rm=TRUE)*0.1/12;

		if(adjusted==TRUE)
		{
			qalys1<-weighted.mean(qalys[subjects[[1]]],rct[subjects[[1]],"w"]);
			coeffs<-coefficients(lm(qalys ~ txgroupf+tempU[,1], data=cbind(qalys, tempU[,1], fRct),weights=rct[,"w"]));
			meanU<-c(qalys1,qalys1+coeffs['txgroupf2'],qalys1+coeffs['txgroupf3']);	
		}
		else
			meanU<-c(weighted.mean(qalys[subjects[[1]]],rct[subjects[[1]],"w"]),weighted.mean(qalys[subjects[[2]]],rct[subjects[[2]],"w"]),weighted.mean(qalys[subjects[[3]]],rct[subjects[[3]],"w"]));
		meanC<-c(weighted.mean(costs[subjects[[1]]],rct[subjects[[1]],"w"]),weighted.mean(costs[subjects[[2]]],rct[subjects[[2]],"w"]),weighted.mean(costs[subjects[[3]]],rct[subjects[[3]],"w"]));
		meanE<-c(weighted.mean(rowSums(tempE[subjects[[1]],],na.rm=TRUE),rct[subjects[[1]],"w"]),weighted.mean(rowSums(tempE[subjects[[2]],],na.rm=TRUE),rct[subjects[[2]],"w"]),weighted.mean(rowSums(tempE[subjects[[3]],],na.rm=TRUE),rct[subjects[[3]],"w"]));
		meanU1<-c(weighted.mean(tempU[subjects[[1]],1],rct[subjects[[1]],"w"]),weighted.mean(tempU[subjects[[2]],1],rct[subjects[[2]],"w"]),weighted.mean(tempU[subjects[[3]],1],rct[subjects[[3]],"w"]));

	}
	eNbs<-meanU*wtp-meanC;

	return(list(cost=meanC, qaly=meanU, exac=meanE, meanU1=meanU1, maxENb=max(eNbs),maxD=which.max(eNbs),eNbs=eNbs,w=w));
}




#Weight of external evidence, for T2 vs T1 (deprecated, returns 1);
weighEffTPTS<-function(p1,p2)
{
	mu<--0.073;
	dv<-0.77812907;
	sd<-sqrt(dv);
	return(1) #(dnorm(log(p2/p1),mean=mu,sd=sd)/dnorm(mu,mean=mu,sd=sd));
}


#Weight of external evidence, for T3 vs T1;
weighEffTPTFS<-function(p1,p2)
{
	mu<-log(0.38);
	sd<-(log(0.57)-mu)/1.96
	v<-sd*sd+0.01783 
	return(dnorm(log(p2/p1),mean=mu,sd=sqrt(v))/dnorm(mu,mean=mu,sd=sqrt(v)));
}


#Deprecated, returns 1;
weighAE<-function(ae1,ae2)
{
	return(1);
}


#Calculate propensity score; 
calcPS<-function(model, data)
{
	n<-dim(data)[1];
	x<-fitted.values(glm(model, data=data, family=binomial));
	return(x);
}
calcPSGroup<-function(ps, bins=5)
{
	temp<-as.vector(1+floor(rank(ps)/(length(ps)/bins)));
	temp[temp==(bins+1)]<-bins;
	return(temp);
}




#Generates bootstrap sample by assinging weights from scaled multinomial (ordinary bootstrap) or Dirichlet (Bayesian bootstrap);
genBsWeights<-function(n, type=BS_TYPE_DIRICHLET)
{
	if(type==BS_TYPE_DIRICHLET)
	{
		temp<-sort(runif(n-1));
		return(c(temp,1)-c(0,temp));
	}
	else
	{
		return(as.vector(rmultinom(1,n,rep(1/n,n))/n));
	}
}

genSample<-function(data, size, weights)
{
	data<-as.data.frame(data);
	return(data[sample(1:dim(data)[1], size, replace=TRUE, prob=weights),]);
}







#Calculates incremental cost-effectiveness ratio from PSA data;
icer<-function(outcomes, d1,d2,type=TYPE_VETTED,level=0)
{
	
	if(type==TYPE_VETTED)
	{
		if(level==0)
		{
			s<-0;
			level<-1;
		} 
		else s<-1;
		if(level<0)
			accept<-outcomes$u*s<outcomes$w[,-level]/outcomes$w[,-level-1]
		else
			accept<-outcomes$u*s<outcomes$w[,level];
		c1<-mean(outcomes$cost[accept,d1]);
		c2<-mean(outcomes$cost[accept,d2]);
		q1<-mean(outcomes$qaly[accept,d1]);
		q2<-mean(outcomes$qaly[accept,d2]);
		e1<-mean(outcomes$exac[accept,d1]);
		e2<-mean(outcomes$exac[accept,d2]);

		extra<-sum(accept)/dim(outcomes$w)[1];
	}
	else
	{
		if(level==0)
		{
			w<-rep(1,dim(outcomes$w)[1]);
		}
		else
			if(level<0)
				w<-outcomes$w[,-level]/outcomes$w[,-level-1]		
			else
				w<-outcomes$w[,level];		 
		c1<-weighted.mean(outcomes$cost[,d1],w);
		c2<-weighted.mean(outcomes$cost[,d2],w);
		q1<-weighted.mean(outcomes$qaly[,d1],w);
		q2<-weighted.mean(outcomes$qaly[,d2],w);
		e1<-weighted.mean(outcomes$exac[,d1],w);
		e2<-weighted.mean(outcomes$exac[,d2],w);

		extra<-sum(w);
	}

	icerQ<-(c2-c1)/(q2-q1);
	icerE<--(c2-c1)/(e2-e1);
	
	return(list(c1=c1,c2=c2,q1=q1,q2=q2,e1=e1,e2=e2,icerQ=icerQ,icerE=icerE, extra=extra));
}



#Draws cost-effectiveness acceptiblity curve from PSA data;
ceac2x2<-function(WTPs,d1,d2,outcomes, eff=EFF_QALY, sgn=1, bVetted, type='l', thickness=1, pch=21, lty=1)
{
	winner<-rep(0,length(WTPs));

	l<-length(acceptBB);
	k<-length(WTPs);
	n<-0;
	for(i in 1:l)
	{
		if(acceptBB[i]>=bVetted)
		{
			n<-n+1;
			for(j in 1:k)
			{
				nb<-sgn*resBB[[i]][[outcome]]*WTPs[j]-resBB[[i]]$cost;
				winner[j]<-winner[j]+((nb[d2]-nb[d1])>0);
			}
		}
	}
	winner<-winner/n;

	plot(WTPs/1000,winner,xlab='willingness-to-pay (X1000)', ylab='probability of cost-effectiveness',ylim=c(0,1),type=type, pch=pch,lty=lty,lwd=thickness);
	return(winner);
}





main<-function(nSim=10000)
{
	init();
	
	n<-dim(rct)[1];
	nArms<-length(unique(rct[,"txgroup"]));
	ns<-rep(0,nArms);
	subjects<-list();

	for(i in 1:nArms)
	{
		subjects<-c(subjects,0);	
		ns[i]<-sum(rct[,"txgroup"]==i);
		subjects[[i]]<-(1:n)[rct[,"txgroup"]==i];
	}

	rctBS<-rct;

	resBB<<-rep(list(NULL),nSim);
	resAB<<-rep(list(NULL),nSim);

	acceptBB<<-rep(0,nSim);
	acceptAB<<-rep(0,nSim);

	uBB<<-rep(0,nSim);
	uAB<<-rep(0,nSim);

	counterBB<<-0;
	counterAB<<-0;

	while(sum(acceptBB)<nSim || sum(acceptAB)<nSim)
	{
		u<-runif(1);

		if(sum(acceptBB)<nSim)
		{
			counterBB<<-counterBB+1;
			if(counterBB>length(resBB))
			{
				resBB<<-c(resBB,rep(list(NULL),nSim));
				acceptBB<<-c(acceptBB,rep(0,nSim));
				uBB<<-c(uBB,rep(0,nSim));
			}
		
			for(i in 1:3)
				rctBS[subjects[[i]],"w"]<-rgamma(ns[i],1);
			resBB[[counterBB]]<<-cea(rct=rctBS);

			uBB[counterBB]<<-u;
	
			if(uBB[counterBB]<resBB[[counterBB]]$w[4])	acceptBB[counterBB]<<-1;

		}

		if(sum(acceptAB)<nSim)
		{
			counterAB<<-counterAB+1;
			if(counterAB>length(resAB))
			{
				resAB<<-c(resAB,rep(list(NULL),nSim));
				acceptAB<<-c(acceptAB,rep(0,nSim));
				uAB<<-c(uAB,rep(0,nSim));
			}
		
			for(i in 1:3)
				rctBS[subjects[[i]],"w"]<-rmultinom(1,ns[i],rep(1,ns[i]));
			resAB[[counterAB]]<<-cea(rct=rctBS);

			uAB[counterAB]<<-u;
	
			if(uAB[counterAB]<resAB[[counterAB]]$w[4])	acceptAB[counterAB]<<-1;

		}
		message(paste(counterAB,'acceptAB=',sum(acceptAB),counterBB,'acceptBB=',sum(acceptBB)));

	}

	resBB<<-resBB[1:counterBB];
	acceptBB<<-acceptBB[1:counterBB];
	resAB<<-resAB[1:counterAB];
	acceptAB<<-acceptAB[1:counterAB];

	uBB<<-uBB[1:counterBB];
	uAB<<-uAB[1:counterAB];

	outcomesBB<<-resBB[[1]];
	for(i in 1:length(outcomesBB))
	{
		outcomesBB[[i]]<<-matrix(0,counterBB,length(outcomesBB[[i]]));
	}
	for(i in 1:length(resBB))
		for(j in 1:length(resBB[[1]]))
			outcomesBB[[j]][i,]<<-resBB[[i]][[j]];
	outcomesBB$u<<-uBB;

	outcomesAB<<-resAB[[1]];
	for(i in 1:length(outcomesAB))
	{
		outcomesAB[[i]]<<-matrix(0,counterAB,length(outcomesAB[[i]]));
	}
	for(i in 1:length(resAB))
		for(j in 1:length(resAB[[1]]))
			outcomesAB[[j]][i,]<<-resAB[[i]][[j]];
	outcomesAB$u<<-uAB;


	#fileName<-paste("./output/",round(runif(1,1,10000000000)),".txt",sep="")
	#save(ans,file=fileName, ascii=TRUE);

	main_vetted_results<<-c(outcomesAB,outcomesBB);
}


MESSAGE<-1;
myMessage<-function(...)
{
	if(MESSAGE>0) 
		message(...);
}






